/************************************************************************************//**
* \file         mainwindow.cpp
* \brief        Application's main window source file.
*
****************************************************************************************/

/****************************************************************************************
* Include files
****************************************************************************************/
#include "mainwindow.h"
#include <wx/xml/xml.h>
#include <wx/stdpaths.h>
#include <wx/filename.h>


/************************************************************************************//**
** \brief     Constructor of the application's main window.
** \param     title Window title.
** \param     width Width of the window in pixels.
** \param     height Height of the window in pixels.
**
****************************************************************************************/
MainWindow::MainWindow(const wxString& title, int width, int height)
  : wxFrame(NULL, wxID_ANY, title, wxDefaultPosition, wxSize(width, height))
{
  /* Center the window on the screen. */
  Centre();

  /* Bind the events to their event handlers. */
  Bind(wxEVT_CLOSE_WINDOW, &MainWindow::onClose, this);  

  /* Load and restore the settings from the XML file. */
  restoreSettings();
} /*** end of MainWindow ***/


/************************************************************************************//**
** \brief     Saves the main window settings to an XML file.
**
****************************************************************************************/
void MainWindow::saveSettings(void)
{
  wxXmlDocument xmlDoc;
  wxString settingsFile = wxStandardPaths::Get().GetUserDataDir() + 
                          wxFileName::GetPathSeparator() + 
                          wxT("settings.xml");
  
  /* Make sure the directory for the settings file actually exists. */
  if (!wxFileName::DirExists(wxStandardPaths::Get().GetUserDataDir()))
  {
    /* Create the directory. */
    wxFileName::Mkdir(wxStandardPaths::Get().GetUserDataDir(), 
                      wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL);
  }

  /* Create and set the root node of the XML file. */
  xmlDoc.SetRoot(new wxXmlNode(wxXML_ELEMENT_NODE, wxT("settings")));

  /* Create the main window child node. */
  wxXmlNode * mainWindowNode = new wxXmlNode(xmlDoc.GetRoot(),
                                             wxXML_ELEMENT_NODE, 
                                             wxT("mainwindow"));

  /* Get the dimensions of the main window. */
  wxSize windowSize = GetSize();

  /* Add the width child node with its value. */
  wxXmlNode * widthNode = new wxXmlNode(mainWindowNode, 
                                        wxXML_ELEMENT_NODE, 
                                        wxT("width"));
  new wxXmlNode(widthNode, wxXML_TEXT_NODE, wxT("width"), 
                wxString::Format(wxT("%i"), windowSize.GetWidth()));

  /* Add the height child node with its value. */
  wxXmlNode * heightNode = new wxXmlNode(mainWindowNode, 
                                         wxXML_ELEMENT_NODE, 
                                         wxT("height"));
  new wxXmlNode(heightNode, wxXML_TEXT_NODE, wxT("height"), 
                wxString::Format(wxT("%i"), windowSize.GetHeight()));

  /* Save the XML settings file. */
  xmlDoc.Save(settingsFile);  
} /*** end of saveSettings ***/


/************************************************************************************//**
** \brief     Reads the main window settings from an XML file.
**
****************************************************************************************/
void MainWindow::restoreSettings(void)
{
  wxXmlDocument xmlDoc;
  wxString settingsFile = wxStandardPaths::Get().GetUserDataDir() + 
                          wxFileName::GetPathSeparator() + 
                          wxT("settings.xml");

  /* Attempt to open the user configuration file as an XML file. */
  if (!xmlDoc.Load(settingsFile))
  {
    return;
  }

  /* Verify that the XML file contains a root node. */
  if (xmlDoc.GetRoot() == nullptr)
  {
     return;
  }

  /* Verify that the root node is the expected settings node. */
  if (!xmlDoc.GetRoot()->GetName().IsSameAs(wxT("settings")))
  {
    return;
  }

  /* Attempt to locate the main window child node. */
  wxXmlNode * mainWindowNode = nullptr;
  wxXmlNode * node = xmlDoc.GetRoot()->GetChildren();
  while (node != nullptr)
  {
    if (node->GetName().IsSameAs(wxT("mainwindow")))
    {
      /* Found our node. Store it and stop the loop. */
      mainWindowNode = node;
      break;
    }
    /* Continue with the next node. */
    node = node->GetNext();
  }

  /* Only continue if we found the main window child node. */
  if (mainWindowNode != nullptr)
  {
    /* Start with the current dimensions of the main window. */
    wxSize windowSize = GetSize();

    /* Loop over all its child nodes. */
    wxXmlNode * node = mainWindowNode->GetChildren();
    while (node != nullptr)
    {
      /* Is this the width node? */
      if (node->GetName().IsSameAs(wxT("width")))
      {
        /* Extract the value. */
        wxString widthStr = node->GetNodeContent();
        if (widthStr.IsNumber())
        {
          windowSize.SetWidth(wxAtoi(widthStr));
        }
      }

      /* Is this the height node? */
      if (node->GetName().IsSameAs(wxT("height")))
      {
        /* Extract the value. */
        wxString heightStr = node->GetNodeContent();
        if (heightStr.IsNumber())
        {
          windowSize.SetHeight(wxAtoi(heightStr));
        }
      }

      /* Continue with the next node. */
      node = node->GetNext();
    }

    /* Restore the dimension of the main window. */
    SetSize(windowSize);
  }  
} /*** end of restoreSettings ***/


/************************************************************************************//**
** \brief     Event handler called when the application's main window closes.
** \param     event Close event object with details about the triggered event.
**
****************************************************************************************/
void MainWindow::onClose(wxCloseEvent& event)
{
  /* Save the application settings. */
  saveSettings();
  
  /* Close the window. */
  Destroy();
} /*** end of onClose ***/


/*********************************** end of mainwindow.cpp *****************************/

